import com.threed.jpct.Matrix;
import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureInfo;
import com.threed.jpct.TextureManager;

/**
 * A decal is what remains if a bullets hits the ground. Decals are managed by
 * the ProjectileManager.
 */
public class Decal extends AbstractEntity {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1094467380350790144L;
	private static long counter = 0;
	private static Object3D DECAL = null;
	private long id = 0;

	static {
		/**
		 * Load all the textures required by this entity and initialize a
		 * "blue print" for decals.
		 */
		Texture tex = new Texture("textures/decal.jpg");
		TextureManager.getInstance().addTexture("decal", tex);

		TextureInfo ti = new TextureInfo(TextureManager.getInstance().getTextureID("decal"));
		ti.add(TextureManager.getInstance().getTextureID("spotlight"), TextureInfo.MODE_MODULATE);

		DECAL = Primitives.getPlane(1, 30);
		DECAL.invert();
		DECAL.setTexture(ti);
		DECAL.getMesh().compress();
		DECAL.setTransparency(3);
		DECAL.build();
	}

	/**
	 * A decal is a plane with a partially transparent texture.
	 */
	public Decal() {
		super(DECAL);
		setVisibility(false);
	}

	/**
	 * A shortcut to Object3D's setRotationMatrix
	 * 
	 * @param m
	 *            the rotation matrix
	 */
	public void rotate(Matrix m) {
		setRotationMatrix(m);
	}

	/**
	 * Places the decal
	 * 
	 * @param pos
	 *            the position of the decal
	 */
	public void place(SimpleVector pos) {
		setVisibility(true);
		setOrigin(pos);
		id = counter;
		counter++;
	}

	/**
	 * Returns the id. The id is used to determine the age of a decal.
	 * 
	 * @return the id
	 */
	public long getDecalID() {
		return id;
	}
}
