import java.awt.Color;

import com.threed.jpct.Loader;
import com.threed.jpct.Logger;
import com.threed.jpct.Matrix;
import com.threed.jpct.Object3D;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureInfo;
import com.threed.jpct.TextureManager;

/**
 * A plant has to be placed on the ground and can be shot.
 */
public class Plant extends AbstractEntity {

	private static final long serialVersionUID = 482184722036415138L;
	private final static Object3D PLANT;
	private boolean dead = false;

	private static TextureInfo plantTi = null;
	private static TextureInfo plantTi2 = null;

	static {
		/**
		 * Load all the textures required by this entity and initialize a
		 * "blue print" for plants.
		 */

		TextureManager texMan = TextureManager.getInstance();

		Texture tex = new Texture("textures/tree.jpg");
		texMan.addTexture("plant", tex);
		Texture tex2 = new Texture("textures/tree2.jpg");
		texMan.addTexture("plant2", tex2);
		Object3D[] objs = Loader.load3DS("models/tree.3ds", 4.5f);

		plantTi = new TextureInfo(texMan.getTextureID("plant"));
		plantTi.add(texMan.getTextureID("spotlight"), TextureInfo.MODE_MODULATE);

		plantTi2 = new TextureInfo(texMan.getTextureID("plant2"));
		plantTi2.add(texMan.getTextureID("spotlight"), TextureInfo.MODE_MODULATE);

		if (objs.length == 1) {
			PLANT = objs[0];
			PLANT.setTexture(plantTi);
			PLANT.setTransparency(200);
			PLANT.setCulling(Object3D.CULLING_DISABLED);
			PLANT.rotateX(-(float) Math.PI);
			PLANT.rotateMesh();
			PLANT.setRotationMatrix(new Matrix());
			PLANT.setAdditionalColor(new Color(100, 100, 100));
			PLANT.build();
		} else {
			// Opps...this shouldn't happen...
			Logger.log("Wrong file format for plants!", Logger.ERROR);
			PLANT = null;
		}
	}

	/**
	 * Creates a new plant.
	 */
	public Plant() {
		super(PLANT);
		this.setMesh(PLANT.getMesh());
		this.shareCompiledData(PLANT);
		this.compile();
		this.build();
		setCollisionMode(Object3D.COLLISION_CHECK_OTHERS);
		setCollisionOptimization(Object3D.COLLISION_DETECTION_OPTIMIZED);
	}

	/**
	 * Place the plant. Placing in this case means that the plant will be
	 * dropped to the ground from it's current position (which has to be above
	 * the ground).
	 * 
	 * @param ground
	 *            the ground
	 * @return success...or not
	 */
	public boolean place(Object3D ground) {

		SimpleVector dropDown = new SimpleVector(0, 1, 0);
		float height = ground.calcMinDistance(getTransformedCenter(), dropDown, 500);

		if (height != Object3D.COLLISION_NONE) {
			dropDown.scalarMul(height + getCenter().y + 1);
			translate(dropDown);
			rotateY((float) Math.random() * 3f);
			enableLazyTransformations();
			return true;
		} else {
			return false;
		}
	}

	/**
	 * "Destroys" the plant. The only thing that happens when destroying a plant
	 * is, that the texture changes and the plant can no longer be hit by
	 * bullets
	 */
	public void destroy() {
		setTexture(plantTi2);
		dead = true;
		setCollisionMode(Object3D.COLLISION_CHECK_NONE);
		disableCollisionListeners();
	}

	/**
	 * Returns is the plant is dead, i.e. if it has been destroyed
	 * 
	 * @return dead or not
	 */
	public boolean isDead() {
		return dead;
	}
}
